#!/usr/bin/env python
# -*- coding:utf-8 -*-
from inspect import getfullargspec
from typing import TypeVar, Dict, List

from .._argument._method_helper import get_func_params

T = TypeVar("T", bound=Dict)
R = TypeVar("R")


def around_impl(func: callable, befores: List[callable] or callable = None, afters: List[callable] or callable = None,
                catch: bool = False, deliver: bool = True, *args, **kwargs):
    copy_kwargs = _copy(kwargs)
    all_func_result = _run_hook_func(func, befores, deliver, args, kwargs)
    __build_origin_params(func, copy_kwargs, all_func_result)
    tmp = []
    # noinspection PyBroadException
    try:
        result = func(*args, **copy_kwargs)
        tmp.append(result)
        return result
    except BaseException as e:
        if not catch:
            raise e
    finally:
        func_result = None
        if tmp:
            func_result = tmp[0]
        if not deliver:
            all_func_result[func.__name__] = func_result
        func_return = copy_kwargs.get("func_return")
        copy_kwargs = _copy(kwargs)
        _run_hook_func(func, afters, deliver, args, copy_kwargs, func_return)


def _run_hook_func(origin_func: callable, call_obj: List[callable] or callable, deliver: bool,
                   origin_args: tuple = None, origin_kwargs: dict = None, func_return: dict = None) -> T:
    result_map = {}
    origin_new_kwargs = get_func_params(origin_func, origin_args, origin_kwargs)
    if not call_obj:
        return {}
    if isinstance(call_obj, list):
        for call in call_obj:
            if callable(call):
                tmp_params = {}
                __hook_params(call, tmp_params, origin_new_kwargs, func_return)
                result_map[call.__name__] = call(**tmp_params)
    elif callable(call_obj):
        result_map[call_obj.__name__] = call_obj()
    if deliver:
        keys = list(result_map.keys())
        if keys:
            return result_map[keys[len(keys) - 1]]
        else:
            return R
    return result_map


def __hook_params(call: callable, params_map: dict, call_func_params: dict, func_return: dict = None):
    args_spec = getfullargspec(call)
    hook_func_all_args = []
    if args_spec.kwonlyargs:
        hook_func_all_args += args_spec.kwonlyargs
    if args_spec.kwonlydefaults:
        hook_func_all_args += list(args_spec.kwonlydefaults.keys())
    hook_func_all_args = set(hook_func_all_args)
    if "func_params" in hook_func_all_args:
        params_map["func_params"] = call_func_params
    if func_return and "func_return" in hook_func_all_args:
        params_map["func_return"] = func_return


def __build_origin_params(call: callable, params_map: dict, call_func_params: dict):
    args_spec = getfullargspec(call)
    hook_func_all_args = []
    if args_spec.kwonlyargs:
        hook_func_all_args += args_spec.kwonlyargs
    if args_spec.kwonlydefaults:
        hook_func_all_args += list(args_spec.kwonlydefaults.keys())
    hook_func_all_args = set(hook_func_all_args)
    if "func_return" in hook_func_all_args:
        params_map["func_return"] = call_func_params


def _copy(values: tuple or dict) -> tuple or dict:
    if isinstance(values, tuple):
        tmp = []
        tmp_append = tmp.append
        for v in values:
            tmp_append(v)
        return tuple(tmp)
    else:
        tmp = {}
        for k, v in values.items():
            tmp[k] = v
        return tmp
