#!/usr/bin/env python
# -*- coding:utf-8 -*-
from functools import wraps

from .._argument._method_helper import run_call_back
from ..exceptions import BasicException


def capture(exception: Exception = BasicException(), call: callable = None):
    """
    Catch exceptions to decorated functions, including exceptions generated by other decorators.
    :param call: while callback function when happen exception
    :param exception: The base type exception to the exception that needs to be caught. After capture,
                    the exception will be "eaten" directly, and exception outside this range will be thrown.
    :return:
    """

    def _inner(func):
        @wraps(func)
        def _wrapper(*args, **kwargs):
            result = None
            try:
                result = func(*args, **kwargs)
                return result
            except BaseException as e:
                if issubclass(type(e), type(exception)):
                    if str(exception).strip():
                        raise exception
                    raise
            finally:
                run_call_back(call, func, args, kwargs, result)
        return _wrapper

    return _inner


def copy(values: tuple, func: callable) -> tuple:
    class_name = func.__qualname__.split(".")[0]
    tmp = []
    tmp_append = tmp.append
    for v in values:
        if v.__class__.__name__ == class_name:
            continue
        tmp_append(v)
    return tuple(tmp)
