#!/usr/bin/env python
# -*- coding:utf-8 -*-
from functools import wraps
from time import time, sleep
from typing import Type

from .._argument._method_helper import run_call_back
from ..log.factory import LoggerFactory

_logger = LoggerFactory.get_logger()


def retry(timeout: int = 60, interval: int = 1, increasing: int = 0, call_back: callable = None,
          ignore_exception: Type[Exception] = None):
    """
    Provides retry functionality.
    unit second
    :param ignore_exception: If it is the exception and its child exceptions, no retry is made
    :param call_back: If the original function is not executed,
    the callback function is executed and the result of the callback function is returned,
    arguments to the original function are passed to the callback function
    :param timeout:
    :param interval:
    :param increasing: The incrementing interval
    :return:
    """

    def _inner(func):
        @wraps(func)
        def _wrapper(*args, **kwargs):
            start = time()
            _interval = interval
            num = 1
            while time() - start < timeout:
                # noinspection PyBroadException
                try:
                    return func(*args, **kwargs)
                except BaseException as e:
                    if ignore_exception and issubclass(type(e), ignore_exception):
                        run_call_back(call_back, func, args, kwargs)
                    _logger.info(f"run function '{func.__name__}' fail: {str(e)}. retrying {num} time(s)")
                    sleep(_interval)
                    num += 1
                    _interval += increasing
            else:
                run_call_back(call_back, func, args, kwargs)

        return _wrapper

    return _inner
