#!/usr/bin/env python
# -*- coding:utf-8 -*-
from decimal import Decimal, ROUND_HALF_UP
from typing import Optional, Union

_T = Optional[Union[int, float, str, bool]]


class Integer(int):
    def __new__(cls, num: _T = 0):
        return int.__new__(cls, num)

    def __init__(self, num: _T = 0):
        self.__num = num


class Float(float):
    def __new__(cls, num: _T = 0):
        return float.__new__(cls, num)

    def __init__(self, num: _T = 0):
        self.__num = num

    def round(self, accuracy: int = None) -> float:
        """
        Rounds floating-point types
        """
        if isinstance(accuracy, int) and accuracy >= 0:
            return float(Decimal(self.__num).quantize(Decimal(f'0.{"0" * accuracy}'), rounding=ROUND_HALF_UP))
        return self.__num

    def integer(self) -> int:
        """
        Output as int type
        """
        return int(self.__num)


__all__ = [Integer, Float]
